/* Header unit for Gipals32.dll.

 There are two possible ways to work with the library Gipals32.dll.

 1) First way is a linear program construction by the provided set of functions,
    such as specification of the variables and constraints count and assignment
    of the constraint matrix elements. This way provides a complete flexibility
    of the linear programming modeling.

 2) The second way is importing linear program from a text file in MPS format.
    It can be done by one function LoadFromMPS. This way can be suitable if there
    is a set of prepared MPS files that need to be calculated.


   Optimalon Software, 2005
*/

// Constraint sign
const  sign_Less  = 0;  // <=
const  sign_Equal = 1;  // =
const  sign_More  = 2;  // >=

// Preprocessor flags
const  prep_Disable          = 0x00;  // Disable preprocessor
const  prep_RemoveFixedVars  = 0x01;  // Remove all fixed variables
const  prep_RemoveEmptyCols  = 0x02;  // Remove empty columns
const  prep_RemoveSingular   = 0x04;  // Remove singular rows and columns
const  prep_RemoveLinearRows = 0x08;  // Remove linear-dependent rows
const  prep_RemoveLinearCols = 0x10;  // Remove linear-dependent columns
const  prep_ForcedRows       = 0x20;  // Remove forced rows

// Calculation result state (negative - unsuccessful, positive - successful)
const  calc_None          =  0;  // Calculation has not been provided yet
const  calc_Optimal       =  1;  // Finished with optimal solution
const  calc_Infeasible    = -1;  // Failed with primal infeasibility
const  calc_DblInfeasible = -2;  // Failed with dual infeasibility
const  calc_Unknown       =  2;  // Finished because reached the max. of iterations
const  calc_Suboptimal    =  3;  // Finished due to slow convergence
const  calc_Stoped        =  4;  // Stoped by the user
const  calc_Error         = -3;  // Failed due to numerical eror
const  calc_Unbounded     = -4;  // Failed because the linear program is unbounded

// Flags returned by the user-defined callback function
const  res_Abort = -1;  // Calculation is canceled
const  res_None  =  0;  // Calculation continues
const  res_Stop  =  1;  // Calculation is stoped and the results are ready

#define DllImport __declspec(dllimport)


// Callback function to control each iteration
typedef int __stdcall ProcessFunc (int IterCount,     // Current iteration
                                   double PrimalObj,  // Primal objective function
                                   double DualObj,    // Dual objective function
                                   double InfPrimal,  // Primal infeasibility
                                   double InfDual,    // Dual infeasibility 
                                   double Optimality);// Optimality 


extern "C" {

// Set number of conastraints (rows)
void DllImport __stdcall SetConstraintCount(int ACount); 

// Set number of decision (primal) variables
void DllImport __stdcall SetVariableCount(int ACount);  

// Returns number of constraints
int DllImport __stdcall GetConstraintCount(void); 

// Returns number of decision (primal) variables
int DllImport __stdcall GetVariableCount(void);

/* Clears the linear program. Deletes all existing data and set number of 
   constraints and variables to 1 */
void DllImport __stdcall ClearLinearProgram(void); 

// Set the objective to minimize (1) or maximize (0)
void DllImport __stdcall SetObjectiveType(int Minimization); 

// Set the cost and bounds for the variable Index. Returns 0 if variable does not exist.
int DllImport __stdcall SetVariable(
			                    int Index,      // Variable index
                                double Cost,    // Objective function coefficient 
                                double Lower,   // Lower bound of variable (ignored if LowerInf <> 0)
                                double Upper,   // Upper bound of variable (ignored if UpperInf <> 0)
                                int LowerInf,   // If <> 0 then indicates variable has no lower bound 
                                int UpperInf);  // If <> 0 then indicates variable has no upper bound 

/* Add a new variable and set the cost and bounds for the variable Index. 
   Returns a new variable index  if succeeded and -1 if the maximum size reached. */
int DllImport __stdcall AddVariable(
								double Cost,    // Objective function coefficient 
                                double Lower,   // Lower bound of variable (ignored if LowerInf <> 0)
                                double Upper,   // Upper bound of variable (ignored if UpperInf <> 0)
                                int LowerInf,   // If <> 0 then indicates variable has no lower bound 
                                int UpperInf);  // If <> 0 then indicates variable has no upper bound 

/* Set the right side and the sign of the specified constraint Index.
   Returns 0 if constraint does not exist. */
int DllImport __stdcall SetConstraint(
								  int Index,    // Index of the constraint 
                                  double Right, // Right side of the constraint
                                  int Sign);    // Sign (sign_Less, sign_Equal, sign_More)

/* Add a new constraint and set its right side and the sign.
   Returns index of the constraint or -1 if the maximum size reached. */
int DllImport __stdcall AddConstraint(
								  double Right, // Right side of the constraint
                                  int Sign);    // Sign (sign_Less, sign_Equal, sign_More)

/* Set the coefficient for the specified variable VarIndex in the specified 
   constraint RowIndex. Returns 1 if succeeded. */
int DllImport __stdcall SetConstraintElement(
										 int RowIndex,  // constraint index
                                         int VarIndex,  // variable index
                                         double Value); // coefficient value


/* The following two routines allow to fill the constraint matrix directly 
   with much higher speed than SetConstraintElement function */

// Selects the constraints to fill. Returns 0 if the constraints does not exist
int DllImport __stdcall DirectSelectConstraint(int Index); 

/* Add a new constraint element in fast way. Variable index MUST be in an 
   ascending order. Returns 1 if succeeded. */
int DllImport __stdcall DirectAddConstraintElement(int Index,     // variable index
                                               double Value); // variable coefficient


/* Load Linear program from the specified file in MPS format.
   Returns: 1 if succeeded. 
            0 if the file is in wrong format
           -1 if the maximum allowed number of variables or constraint reached */
int DllImport __stdcall LoadFromMPS(char* AFile);


/* The main function. Calculates the linear program and returns result.
   The result is defined in "Calculation result" section above.
   ProcessFunc is callbac function provided to control iteration and cancel
   or stop the calculation.  */ 
int DllImport __stdcall Calculate(ProcessFunc ProcessCallback);


// GET RESULTS
// Returns primal objective function value
double DllImport __stdcall PrimalObjective(void); 

// Returns dual objective function value
double DllImport __stdcall DualObjective(void);

// Returns value of the specified decision (primal) variable (X)
double DllImport __stdcall PrimalValue(int Index);

// Returns value of the specified dual variable (Y)
double DllImport __stdcall DualValue(int Index);

// Returns value of the specified reduced cost variable (S)
double DllImport __stdcall ReducedCostValue(int Index);


/////////////////////////////////////////////////////////////////////////////////////////////
// The following routines are optional and should be used to tune the calculation engine.  //
/////////////////////////////////////////////////////////////////////////////////////////////

// Flags are combination of Preprocessor flags or prep_Disable to disable
void DllImport __stdcall SetPreprocessor(int Flags);

// Set tolerances. Defaults are: Primal=1e-8, Dual=1e-8, Optimal=1e-10
void DllImport __stdcall SetTolerances(double Primal, double Dual, double Optimal);

// Set maximum of iterations (default is 100)
void DllImport __stdcall SetMaxIterationCount(int Value);

// Enable/Disable the LP scaling. 1 - enable (default), 0 - disable.
void DllImport __stdcall SetScaling(int Value); 

// Enable/Disable the iterative refinement. 1 - enable (default), 0 - disable.
void DllImport __stdcall SetRefinement(int Value);

// Use the Gondzio correction. 1 - use (default), 0 - don't.
void DllImport __stdcall SetGondzio(int Value); 
}